@@ -13,10 +13,11 @@ from .cwt import (
 from .encrypted_cose_key import EncryptedCOSEKey
 from .exceptions import CWTError, DecodeError, EncodeError, VerifyError
 from .helpers.hcert import load_pem_hcert_dsc
+from .hpke_cipher_suite import HPKECipherSuite
 from .recipient import Recipient
 from .signer import Signer
 
-__version__ = "1.5.7"
+__version__ = "1.6.0"
 __title__ = "cwt"
 __description__ = "A Python implementation of CWT/COSE"
 __url__ = "https://python-cwt.readthedocs.io"
@@ -37,6 +38,7 @@ __all__ = [
     "COSE",
     "COSEKey",
     "EncryptedCOSEKey",
+    "HPKECipherSuite",
     "Claims",
     "Recipient",
     "Signer",
