@@ -120,6 +120,19 @@ class ECDH_DirectHKDF(Direct):
             raise ValueError("context should be set.")
         return key.derive_key(context, public_key=self._sender_public_key)
 
+    def decrypt(
+        self,
+        key: COSEKeyInterface,
+        alg: Optional[int] = None,
+        context: Optional[Union[List[Any], Dict[str, Any]]] = None,
+        payload: bytes = b"",
+        nonce: bytes = b"",
+        aad: bytes = b"",
+        external_aad: bytes = b"",
+        aad_context: str = "Enc_Recipient",
+    ) -> bytes:
+        return self.extract(key, alg, context).decrypt(payload, nonce, aad)
+
     def _apply_context(self, given: list) -> list:
         ctx = copy.deepcopy(self._default_ctx)
         for i, item in enumerate(given):
