@@ -132,10 +132,13 @@ class RecipientInterface(CBORProcessor):
         """
         return self._recipients
 
-    def to_list(self) -> List[Any]:
+    def to_list(self, payload: bytes = b"", external_aad: bytes = b"", aad_context: str = "") -> List[Any]:
         """
         Returns the recipient information as a COSE recipient structure.
 
+        Args:
+            payload (Optional[bytes]): The payload to be encrypted.
+            external_aad (Optional[bytes]): External additional authenticated data.
         Returns:
             List[Any]: The recipient structure.
         """
@@ -147,7 +150,7 @@ class RecipientInterface(CBORProcessor):
 
         children = []
         for recipient in self._recipients:
-            children.append(recipient.to_list())
+            children.append(recipient.to_list(payload, external_aad, aad_context))
         res.append(children)
         return res
 
@@ -172,7 +175,7 @@ class RecipientInterface(CBORProcessor):
             key (Optional[COSEKeyInterface]): The external key to
                 be used for preparing the key.
             recipient_key (Optional[COSEKeyInterface]): The external public
-                key provided by the recipient used for ECDH key agreement.
+                key provided by the recipient used for ECDH key agreement, etc.
             salt (Optional[bytes]): A salt used for deriving a key.
             context (Optional[Union[List[Any], Dict[str, Any]]]): Context
                 information structure.
@@ -212,6 +215,33 @@ class RecipientInterface(CBORProcessor):
         """
         raise NotImplementedError
 
+    def decrypt(
+        self,
+        key: COSEKeyInterface,
+        alg: Optional[int] = None,
+        context: Optional[Union[List[Any], Dict[str, Any]]] = None,
+        payload: bytes = b"",
+        nonce: bytes = b"",
+        aad: bytes = b"",
+        external_aad: bytes = b"",
+        aad_context: str = "Enc_Recipient",
+    ) -> bytes:
+        """
+        Decrypts the supplied payload.
+
+        Args:
+            key (COSEKeyInterface): The external key to be used for extracting the key.
+            alg (Optional[int]): The algorithm of the key extracted.
+            context (Optional[Union[List[Any], Dict[str, Any]]]): Context information structure.
+            external_aad (bytes): External additional authenticated data.
+        Returns:
+            bytes: The decrypted plain text.
+        Raises:
+            ValueError: Invalid arguments.
+            DecodeError: Failed to decode(e.g., unwrap, derive) the key.
+        """
+        raise NotImplementedError
+
     def _to_cose_key(self, k: Union[EllipticCurvePublicKey, X25519PublicKey, X448PublicKey]) -> Dict[int, Any]:
         if isinstance(k, EllipticCurvePublicKey):
             return EC2Key.to_cose_key(k)
