@@ -1,5 +1,7 @@
 from secrets import token_bytes
 
+import pytest
+
 from cwt import COSE, COSEKey, Recipient, Signer
 
 
@@ -11,48 +13,58 @@ class TestCOSESample:
     def test_cose_usage_examples_cose_mac0(self):
         mac_key = COSEKey.from_symmetric_key(alg="HS256", kid="01")
 
-        ctx = COSE.new(alg_auto_inclusion=True, kid_auto_inclusion=True)
-        encoded = ctx.encode_and_mac(b"Hello world!", mac_key)
-        assert b"Hello world!" == ctx.decode(encoded, mac_key)
+        # The sender side:
+        sender = COSE.new(alg_auto_inclusion=True, kid_auto_inclusion=True)
+        encoded = sender.encode_and_mac(b"Hello world!", mac_key)
 
-        ctx = COSE.new()
-        encoded2 = ctx.encode_and_mac(
+        # The recipient side:
+        recipient = COSE.new()
+        assert b"Hello world!" == recipient.decode(encoded, mac_key)
+
+        # variation
+        sender = COSE.new()
+        encoded2 = sender.encode_and_mac(
             b"Hello world!",
             mac_key,
             protected={"alg": "HS256"},
             unprotected={"kid": "01"},
         )
-        assert b"Hello world!" == ctx.decode(encoded2, mac_key)
+        assert b"Hello world!" == recipient.decode(encoded2, mac_key)
 
-        encoded3 = ctx.encode_and_mac(
+        encoded3 = sender.encode_and_mac(
             b"Hello world!",
             mac_key,
             protected={1: 5},
             unprotected={4: b"01"},
         )
-        assert b"Hello world!" == ctx.decode(encoded3, mac_key)
+        assert b"Hello world!" == recipient.decode(encoded3, mac_key)
 
         assert encoded == encoded2 == encoded3
 
     def test_cose_usage_examples_cose_mac_direct(self):
         mac_key = COSEKey.from_symmetric_key(alg="HS512", kid="01")
 
+        # The sender side:
         r = Recipient.from_jwk({"alg": "direct"})
         r.apply(mac_key)
 
-        ctx = COSE.new()
-        encoded = ctx.encode_and_mac(b"Hello world!", mac_key, recipients=[r])
-        assert b"Hello world!" == ctx.decode(encoded, mac_key)
+        sender = COSE.new()
+        encoded = sender.encode_and_mac(b"Hello world!", mac_key, recipients=[r])
 
+        # The recipient side:
+        recipient = COSE.new()
+        assert b"Hello world!" == recipient.decode(encoded, mac_key)
+
+        # variation
         r2 = Recipient.new(unprotected={"alg": "direct"})
         r2.apply(mac_key)
-        encoded2 = ctx.encode_and_mac(b"Hello world!", mac_key, recipients=[r2])
-        assert b"Hello world!" == ctx.decode(encoded2, mac_key)
+        encoded2 = sender.encode_and_mac(b"Hello world!", mac_key, recipients=[r2])
+        assert b"Hello world!" == recipient.decode(encoded2, mac_key)
 
         r3 = Recipient.new(unprotected={1: -6})
         r3.apply(mac_key)
-        encoded3 = ctx.encode_and_mac(b"Hello world!", mac_key, recipients=[r3])
-        assert b"Hello world!" == ctx.decode(encoded3, mac_key)
+        encoded3 = sender.encode_and_mac(b"Hello world!", mac_key, recipients=[r3])
+        assert b"Hello world!" == recipient.decode(encoded3, mac_key)
 
         assert encoded == encoded2 == encoded3
 
@@ -61,6 +73,7 @@ class TestCOSESample:
         shared_material = token_bytes(32)
         shared_key = COSEKey.from_symmetric_key(shared_material, kid="01")
 
+        # The sender side:
         r = Recipient.from_jwk(
             {
                 "kty": "oct",
@@ -69,13 +82,16 @@ class TestCOSESample:
             },
         )
         mac_key = r.apply(shared_key, context={"alg": "HS256"})
-        ctx = COSE.new(alg_auto_inclusion=True)
-        encoded = ctx.encode_and_mac(
+        sender = COSE.new(alg_auto_inclusion=True)
+        encoded = sender.encode_and_mac(
             b"Hello world!",
             key=mac_key,
             recipients=[r],
         )
-        assert b"Hello world!" == ctx.decode(encoded, shared_key, context={"alg": "HS256"})
+
+        # The recipient side:
+        recipient = COSE.new()
+        assert b"Hello world!" == recipient.decode(encoded, shared_key, context={"alg": "HS256"})
 
     def test_cose_usage_examples_cose_mac_aes_key_wrap(self):
 
@@ -90,10 +106,11 @@ class TestCOSESample:
             },
         )
         r.apply(mac_key)
-        ctx = COSE.new(alg_auto_inclusion=True)
-        encoded = ctx.encode_and_mac(b"Hello world!", key=mac_key, recipients=[r])
+        sender = COSE.new(alg_auto_inclusion=True)
+        encoded = sender.encode_and_mac(b"Hello world!", key=mac_key, recipients=[r])
 
         # The recipient side:
+        recipient = COSE.new()
         shared_key = COSEKey.from_jwk(
             {
                 "kty": "oct",
@@ -102,17 +119,12 @@ class TestCOSESample:
                 "k": "hJtXIZ2uSN5kbQfbtTNWbg",
             },
         )
-        assert b"Hello world!" == ctx.decode(encoded, shared_key)
+        assert b"Hello world!" == recipient.decode(encoded, shared_key)
 
     def test_cose_usage_examples_cose_mac_ecdh_direct_hkdf_p256(self):
 
-        r = Recipient.from_jwk(
-            {
-                "kty": "EC",
-                "alg": "ECDH-ES+HKDF-256",
-                "crv": "P-256",
-            },
-        )
+        # The sender side:
+        # The following key is provided by the recipient in advance.
         pub_key = COSEKey.from_jwk(
             {
                 "kty": "EC",
@@ -122,13 +134,23 @@ class TestCOSESample:
                 "y": "HlLtdXARY_f55A3fnzQbPcm6hgr34Mp8p-nuzQCE0Zw",
             }
         )
+        r = Recipient.from_jwk(
+            {
+                "kty": "EC",
+                "alg": "ECDH-ES+HKDF-256",
+                "crv": "P-256",
+            },
+        )
         mac_key = r.apply(recipient_key=pub_key, context={"alg": "HS256"})
-        ctx = COSE.new(alg_auto_inclusion=True)
-        encoded = ctx.encode_and_mac(
+        sender = COSE.new(alg_auto_inclusion=True)
+        encoded = sender.encode_and_mac(
             b"Hello world!",
             key=mac_key,
             recipients=[r],
         )
+
+        # The recipient side:
+        # The following key is the private key of the above pub_key.
         priv_key = COSEKey.from_jwk(
             {
                 "kty": "EC",
@@ -140,10 +162,15 @@ class TestCOSESample:
                 "d": "r_kHyZ-a06rmxM3yESK84r1otSg-aQcVStkRhA-iCM8",
             }
         )
-        assert b"Hello world!" == ctx.decode(encoded, priv_key, context={"alg": "HS256"})
+        recipient = COSE.new()
+        # The enc_key will be derived in decode() with priv_key and
+        # the sender's public key which is conveyed as the recipient
+        # information structure in the COSE Encrypt message (encoded).
+        assert b"Hello world!" == recipient.decode(encoded, priv_key, context={"alg": "HS256"})
 
     def test_cose_usage_examples_cose_mac_ecdh_direct_hkdf_x25519(self):
 
+        # The sender side:
         r = Recipient.from_jwk(
             {
                 "kty": "OKP",
@@ -161,12 +188,15 @@ class TestCOSESample:
             }
         )
         mac_key = r.apply(recipient_key=pub_key, context={"alg": "HS256"})
-        ctx = COSE.new(alg_auto_inclusion=True)
-        encoded = ctx.encode_and_mac(
+        sender = COSE.new(alg_auto_inclusion=True)
+        encoded = sender.encode_and_mac(
             b"Hello world!",
             key=mac_key,
             recipients=[r],
         )
+
+        # The recipient side:
+        recipient = COSE.new()
         priv_key = COSEKey.from_jwk(
             {
                 "kty": "OKP",
@@ -177,10 +207,11 @@ class TestCOSESample:
                 "d": "vsJ1oX5NNi0IGdwGldiac75r-Utmq3Jq4LGv48Q_Qc4",
             }
         )
-        assert b"Hello world!" == ctx.decode(encoded, priv_key, context={"alg": "HS256"})
+        assert b"Hello world!" == recipient.decode(encoded, priv_key, context={"alg": "HS256"})
 
     def test_cose_usage_examples_cose_mac_ecdh_direct_hkdf_x448(self):
 
+        # The sender side:
         r = Recipient.from_jwk(
             {
                 "kty": "OKP",
@@ -198,12 +229,15 @@ class TestCOSESample:
             }
         )
         mac_key = r.apply(recipient_key=pub_key, context={"alg": "HS256"})
-        ctx = COSE.new(alg_auto_inclusion=True)
-        encoded = ctx.encode_and_mac(
+        sender = COSE.new(alg_auto_inclusion=True)
+        encoded = sender.encode_and_mac(
             b"Hello world!",
             key=mac_key,
             recipients=[r],
         )
+
+        # The recipient side:
+        recipient = COSE.new()
         priv_key = COSEKey.from_jwk(
             {
                 "kty": "OKP",
@@ -214,7 +248,7 @@ class TestCOSESample:
                 "d": "rJJRG3nshyCtd9CgXld8aNaB9YXKR0UOi7zj7hApg9YH4XdBO0G8NcAFNz_uPH2GnCZVcSDgV5c",
             }
         )
-        assert b"Hello world!" == ctx.decode(encoded, priv_key, context={"alg": "HS256"})
+        assert b"Hello world!" == recipient.decode(encoded, priv_key, context={"alg": "HS256"})
 
     def test_cose_usage_examples_cose_mac_ecdh_ss_a128kw(self):
 
@@ -240,14 +274,15 @@ class TestCOSESample:
             }
         )
         r.apply(mac_key, recipient_key=pub_key, context={"alg": "HS256"})
-        ctx = COSE.new(alg_auto_inclusion=True)
-        encoded = ctx.encode_and_mac(
+        sender = COSE.new(alg_auto_inclusion=True)
+        encoded = sender.encode_and_mac(
             b"Hello world!",
             key=mac_key,
             recipients=[r],
         )
 
         # The recipient side:
+        recipient = COSE.new()
         priv_key = COSEKey.from_jwk(
             {
                 "kty": "EC",
@@ -259,77 +294,276 @@ class TestCOSESample:
                 "d": "r_kHyZ-a06rmxM3yESK84r1otSg-aQcVStkRhA-iCM8",
             }
         )
-        assert b"Hello world!" == ctx.decode(encoded, priv_key, context={"alg": "HS256"})
+        assert b"Hello world!" == recipient.decode(encoded, priv_key, context={"alg": "HS256"})
 
     def test_cose_usage_examples_cose_encrypt0(self):
         enc_key = COSEKey.from_symmetric_key(alg="ChaCha20/Poly1305", kid="01")
 
+        # The sender side:
         nonce = enc_key.generate_nonce()
-        ctx = COSE.new(alg_auto_inclusion=True, kid_auto_inclusion=True)
-        encoded = ctx.encode_and_encrypt(b"Hello world!", enc_key, nonce=nonce)
-        assert b"Hello world!" == ctx.decode(encoded, enc_key)
+        sender = COSE.new(alg_auto_inclusion=True, kid_auto_inclusion=True)
+        encoded = sender.encode_and_encrypt(b"Hello world!", enc_key, nonce=nonce)
 
-        ctx = COSE.new()
-        encoded2 = ctx.encode_and_encrypt(
+        # The recipient side:
+        recipient = COSE.new()
+        assert b"Hello world!" == recipient.decode(encoded, enc_key)
+
+        # variation
+        sender = COSE.new()
+        encoded2 = sender.encode_and_encrypt(
             b"Hello world!",
             enc_key,
             nonce=nonce,
             protected={"alg": "ChaCha20/Poly1305"},
             unprotected={"kid": "01"},
         )
-        assert b"Hello world!" == ctx.decode(encoded2, enc_key)
+        assert b"Hello world!" == recipient.decode(encoded2, enc_key)
 
-        encoded3 = ctx.encode_and_encrypt(
+        encoded3 = sender.encode_and_encrypt(
             b"Hello world!",
             enc_key,
             nonce=nonce,
             protected={1: 24},
             unprotected={4: b"01"},
         )
-        assert b"Hello world!" == ctx.decode(encoded3, enc_key)
+        assert b"Hello world!" == recipient.decode(encoded3, enc_key)
 
         assert encoded == encoded2 == encoded3
 
+    def test_cose_usage_examples_cose_encrypt0_hpke(self):
+        # The sender side:
+        rpk = COSEKey.from_jwk(
+            {
+                "kty": "EC",
+                "kid": "01",
+                "crv": "P-256",
+                # "alg": "HPKE",
+                "x": "usWxHK2PmfnHKwXPS54m0kTcGJ90UiglWiGahtagnv8",
+                "y": "IBOL-C3BttVivg-lSreASjpkttcsz-1rb7btKLv8EX4",
+            }
+        )
+
+        sender = COSE.new()
+        encoded = sender.encode_and_encrypt(
+            b"This is the content.",
+            rpk,
+            protected={
+                1: -1,  # alg: "HPKE"
+            },
+            unprotected={
+                4: b"01",  # kid: "01"
+                -4: {  # HPKE sender information
+                    1: 0x0010,  # kem: DHKEM(P-256, HKDF-SHA256)
+                    5: 0x0001,  # kdf: HKDF-SHA256
+                    2: 0x0001,  # aead: AES-128-GCM
+                },
+            },
+        )
+
+        # print(encoded.hex())
+
+        # The recipient side:
+        rsk = COSEKey.from_jwk(
+            {
+                "kty": "EC",
+                "kid": "01",
+                "crv": "P-256",
+                # "alg": "HPKE",
+                "x": "usWxHK2PmfnHKwXPS54m0kTcGJ90UiglWiGahtagnv8",
+                "y": "IBOL-C3BttVivg-lSreASjpkttcsz-1rb7btKLv8EX4",
+                "d": "V8kgd2ZBRuh2dgyVINBUqpPDr7BOMGcF22CQMIUHtNM",
+            }
+        )
+        recipient = COSE.new()
+        assert b"This is the content." == recipient.decode(encoded, rsk)
+
     def test_cose_usage_examples_cose_encrypt(self):
         enc_key = COSEKey.from_symmetric_key(alg="ChaCha20/Poly1305", kid="01")
+
+        # The sender side:
         nonce = enc_key.generate_nonce()
         r = Recipient.from_jwk({"alg": "direct"})
         r.apply(enc_key)
 
-        ctx = COSE.new()
-        encoded = ctx.encode_and_encrypt(
+        sender = COSE.new()
+        encoded = sender.encode_and_encrypt(
             b"Hello world!",
             enc_key,
             nonce=nonce,
             recipients=[r],
         )
-        assert b"Hello world!" == ctx.decode(encoded, enc_key)
 
+        # The recipient side:
+        recipient = COSE.new()
+        assert b"Hello world!" == recipient.decode(encoded, enc_key)
+
+        # variation
         r = Recipient.new(unprotected={"alg": "direct"})
         r.apply(enc_key)
-        encoded2 = ctx.encode_and_encrypt(
+        encoded2 = sender.encode_and_encrypt(
             b"Hello world!",
             enc_key,
             nonce=nonce,
             recipients=[r],
         )
-        assert b"Hello world!" == ctx.decode(encoded2, enc_key)
+        assert b"Hello world!" == recipient.decode(encoded2, enc_key)
 
-        encoded3 = ctx.encode_and_encrypt(
+        encoded3 = sender.encode_and_encrypt(
             b"Hello world!",
             enc_key,
             nonce=nonce,
             recipients=[r],
         )
-        assert b"Hello world!" == ctx.decode(encoded3, enc_key)
+        assert b"Hello world!" == recipient.decode(encoded3, enc_key)
 
         assert encoded == encoded2 == encoded3
 
+    def test_cose_usage_examples_cose_encrypt_hpke(self):
+
+        # The sender side:
+        rpk = COSEKey.from_jwk(
+            {
+                "kty": "EC",
+                "kid": "01",
+                "crv": "P-256",
+                "x": "usWxHK2PmfnHKwXPS54m0kTcGJ90UiglWiGahtagnv8",
+                "y": "IBOL-C3BttVivg-lSreASjpkttcsz-1rb7btKLv8EX4",
+            }
+        )
+        r = Recipient.new(
+            protected={
+                1: -1,  # alg: "HPKE"
+            },
+            unprotected={
+                4: b"01",  # kid: "01"
+                -4: {  # HPKE sender information
+                    1: 0x0010,  # kem: DHKEM(P-256, HKDF-SHA256)
+                    5: 0x0001,  # kdf: HKDF-SHA256
+                    2: 0x0001,  # aead: AES-128-GCM
+                },
+            },
+        )
+        r.apply(recipient_key=rpk)
+        sender = COSE.new()
+        encoded = sender.encode_and_encrypt(
+            b"This is the content.",
+            recipients=[r],
+        )
+
+        # print(encoded.hex())
+
+        # The recipient side:
+        rsk = COSEKey.from_jwk(
+            {
+                "kty": "EC",
+                "kid": "01",
+                "crv": "P-256",
+                "x": "usWxHK2PmfnHKwXPS54m0kTcGJ90UiglWiGahtagnv8",
+                "y": "IBOL-C3BttVivg-lSreASjpkttcsz-1rb7btKLv8EX4",
+                "d": "V8kgd2ZBRuh2dgyVINBUqpPDr7BOMGcF22CQMIUHtNM",
+            }
+        )
+        recipient = COSE.new()
+        assert b"This is the content." == recipient.decode(encoded, rsk)
+
+    def test_cose_usage_examples_cose_encrypt_hpke_with_1st_layer_hpke(self):
+
+        # The sender side:
+        rpk = COSEKey.from_jwk(
+            {
+                "kty": "EC",
+                "kid": "01",
+                "crv": "P-256",
+                "x": "usWxHK2PmfnHKwXPS54m0kTcGJ90UiglWiGahtagnv8",
+                "y": "IBOL-C3BttVivg-lSreASjpkttcsz-1rb7btKLv8EX4",
+            }
+        )
+        r = Recipient.new(
+            protected={
+                1: -1,  # alg: "HPKE"
+            },
+            unprotected={
+                4: b"01",  # kid: "01"
+                -4: {  # HPKE sender information
+                    1: 0x0010,  # kem: DHKEM(P-256, HKDF-SHA256)
+                    5: 0x0001,  # kdf: HKDF-SHA256
+                    2: 0x0001,  # aead: AES-128-GCM
+                },
+            },
+        )
+        r.apply(recipient_key=rpk)
+        sender = COSE.new()
+        with pytest.raises(ValueError) as err:
+            sender.encode_and_encrypt(
+                b"This is the content.",
+                protected={
+                    1: -1,  # alg: "HPKE"
+                },
+                unprotected={
+                    4: b"xx",  # kid: "xx"
+                    -4: {  # HPKE sender information
+                        1: 0x0010,  # kem: DHKEM(P-256, HKDF-SHA256)
+                        5: 0x0001,  # kdf: HKDF-SHA256
+                        2: 0x0001,  # aead: AES-128-GCM
+                    },
+                },
+                recipients=[r],
+            )
+            pytest.fail("encode_and_encrypt should fail.")
+        assert "alg for the first layer should not be HPKE." in str(err.value)
+
+    def test_cose_usage_examples_cose_encrypt_hpke_with_nonce(self):
+
+        # The sender side:
+        rpk = COSEKey.from_jwk(
+            {
+                "kty": "EC",
+                "kid": "01",
+                "crv": "P-256",
+                "x": "usWxHK2PmfnHKwXPS54m0kTcGJ90UiglWiGahtagnv8",
+                "y": "IBOL-C3BttVivg-lSreASjpkttcsz-1rb7btKLv8EX4",
+            }
+        )
+        r = Recipient.new(
+            protected={
+                1: -1,  # alg: "HPKE"
+            },
+            unprotected={
+                4: b"01",  # kid: "01"
+                -4: {  # HPKE sender information
+                    1: 0x0010,  # kem: DHKEM(P-256, HKDF-SHA256)
+                    5: 0x0001,  # kdf: HKDF-SHA256
+                    2: 0x0001,  # aead: AES-128-GCM
+                },
+            },
+        )
+        r.apply(recipient_key=rpk)
+        sender = COSE.new()
+        with pytest.raises(ValueError) as err:
+            sender.encode_and_encrypt(
+                b"This is the content.",
+                protected={
+                    1: -1,  # alg: "HPKE"
+                },
+                unprotected={
+                    4: b"xx",  # kid: "xx"
+                    -4: {  # HPKE sender information
+                        1: 0x0010,  # kem: DHKEM(P-256, HKDF-SHA256)
+                        5: 0x0001,  # kdf: HKDF-SHA256
+                        2: 0x0001,  # aead: AES-128-GCM
+                    },
+                },
+                recipients=[r],
+            )
+            pytest.fail("encode_and_encrypt should fail.")
+        assert "alg for the first layer should not be HPKE." in str(err.value)
+
     def test_cose_usage_examples_cose_encrypt_direct_hkdf_sha_256(self):
 
         shared_material = token_bytes(32)
         shared_key = COSEKey.from_symmetric_key(shared_material, kid="01")
 
+        # The sender side:
         r = Recipient.from_jwk(
             {
                 "kty": "oct",
@@ -338,18 +572,22 @@ class TestCOSESample:
             },
         )
         enc_key = r.apply(shared_key, context={"alg": "A256GCM"})
-        ctx = COSE.new(alg_auto_inclusion=True)
-        encoded = ctx.encode_and_encrypt(
+        sender = COSE.new(alg_auto_inclusion=True)
+        encoded = sender.encode_and_encrypt(
             b"Hello world!",
             key=enc_key,
             recipients=[r],
         )
-        assert b"Hello world!" == ctx.decode(encoded, shared_key, context={"alg": "A256GCM"})
+
+        # The recipient side:
+        recipient = COSE.new()
+        assert b"Hello world!" == recipient.decode(encoded, shared_key, context={"alg": "A256GCM"})
 
     def test_cose_usage_examples_cose_encrypt_aes_key_wrap_a128kw(self):
         # A key to wrap
         enc_key = COSEKey.from_symmetric_key(alg="ChaCha20/Poly1305")
 
+        # The sender side:
         r = Recipient.from_jwk(
             {
                 "kty": "oct",
@@ -359,8 +597,11 @@ class TestCOSESample:
             },
         )
         r.apply(enc_key)
-        ctx = COSE.new(alg_auto_inclusion=True)
-        encoded = ctx.encode_and_encrypt(b"Hello world!", key=enc_key, recipients=[r])
+        sender = COSE.new(alg_auto_inclusion=True)
+        encoded = sender.encode_and_encrypt(b"Hello world!", key=enc_key, recipients=[r])
+
+        # The recipient side:
+        recipient = COSE.new()
         shared_key = COSEKey.from_jwk(
             {
                 "kty": "oct",
@@ -369,10 +610,11 @@ class TestCOSESample:
                 "k": "hJtXIZ2uSN5kbQfbtTNWbg",
             },
         )
-        assert b"Hello world!" == ctx.decode(encoded, shared_key)
+        assert b"Hello world!" == recipient.decode(encoded, shared_key)
 
     def test_cose_usage_examples_cose_encrypt_ecdh_direct_hkdf_p256(self):
 
+        # The sender side:
         r = Recipient.from_jwk(
             {
                 "kty": "EC",
@@ -390,12 +632,15 @@ class TestCOSESample:
             }
         )
         enc_key = r.apply(recipient_key=pub_key, context={"alg": "A128GCM"})
-        ctx = COSE.new(alg_auto_inclusion=True)
-        encoded = ctx.encode_and_encrypt(
+        sender = COSE.new(alg_auto_inclusion=True)
+        encoded = sender.encode_and_encrypt(
             b"Hello world!",
             key=enc_key,
             recipients=[r],
         )
+
+        # The recipient side:
+        recipient = COSE.new()
         priv_key = COSEKey.from_jwk(
             {
                 "kty": "EC",
@@ -407,10 +652,11 @@ class TestCOSESample:
                 "d": "r_kHyZ-a06rmxM3yESK84r1otSg-aQcVStkRhA-iCM8",
             }
         )
-        assert b"Hello world!" == ctx.decode(encoded, priv_key, context={"alg": "A128GCM"})
+        assert b"Hello world!" == recipient.decode(encoded, priv_key, context={"alg": "A128GCM"})
 
     def test_cose_usage_examples_cose_encrypt_ecdh_direct_hkdf_x25519(self):
 
+        # The sender side:
         r = Recipient.from_jwk(
             {
                 "kty": "OKP",
@@ -428,12 +674,15 @@ class TestCOSESample:
             }
         )
         enc_key = r.apply(recipient_key=pub_key, context={"alg": "A128GCM"})
-        ctx = COSE.new(alg_auto_inclusion=True)
-        encoded = ctx.encode_and_encrypt(
+        sender = COSE.new(alg_auto_inclusion=True)
+        encoded = sender.encode_and_encrypt(
             b"Hello world!",
             key=enc_key,
             recipients=[r],
         )
+
+        # The recipient side:
+        recipient = COSE.new()
         priv_key = COSEKey.from_jwk(
             {
                 "kty": "OKP",
@@ -444,10 +693,11 @@ class TestCOSESample:
                 "d": "vsJ1oX5NNi0IGdwGldiac75r-Utmq3Jq4LGv48Q_Qc4",
             }
         )
-        assert b"Hello world!" == ctx.decode(encoded, priv_key, context={"alg": "A128GCM"})
+        assert b"Hello world!" == recipient.decode(encoded, priv_key, context={"alg": "A128GCM"})
 
     def test_cose_usage_examples_cose_encrypt_ecdh_direct_hkdf_x448(self):
 
+        # The sender side:
         r = Recipient.from_jwk(
             {
                 "kty": "OKP",
@@ -465,12 +715,15 @@ class TestCOSESample:
             }
         )
         enc_key = r.apply(recipient_key=pub_key, context={"alg": "A128GCM"})
-        ctx = COSE.new(alg_auto_inclusion=True)
-        encoded = ctx.encode_and_encrypt(
+        sender = COSE.new(alg_auto_inclusion=True)
+        encoded = sender.encode_and_encrypt(
             b"Hello world!",
             key=enc_key,
             recipients=[r],
         )
+
+        # The recipient side:
+        recipient = COSE.new()
         priv_key = COSEKey.from_jwk(
             {
                 "kty": "OKP",
@@ -481,7 +734,7 @@ class TestCOSESample:
                 "d": "rJJRG3nshyCtd9CgXld8aNaB9YXKR0UOi7zj7hApg9YH4XdBO0G8NcAFNz_uPH2GnCZVcSDgV5c",
             }
         )
-        assert b"Hello world!" == ctx.decode(encoded, priv_key, context={"alg": "A128GCM"})
+        assert b"Hello world!" == recipient.decode(encoded, priv_key, context={"alg": "A128GCM"})
 
     def test_cose_usage_examples_cose_encrypt_ecdh_ss_a128kw(self):
 
@@ -508,8 +761,8 @@ class TestCOSESample:
             }
         )
         r.apply(enc_key, recipient_key=pub_key, context={"alg": "A128GCM"})
-        ctx = COSE.new(alg_auto_inclusion=True)
-        encoded = ctx.encode_and_encrypt(
+        sender = COSE.new(alg_auto_inclusion=True)
+        encoded = sender.encode_and_encrypt(
             b"Hello world!",
             key=enc_key,
             nonce=nonce,
@@ -517,6 +770,7 @@ class TestCOSESample:
         )
 
         # The recipient side:
+        recipient = COSE.new()
         priv_key = COSEKey.from_jwk(
             {
                 "kty": "EC",
@@ -528,10 +782,11 @@ class TestCOSESample:
                 "d": "r_kHyZ-a06rmxM3yESK84r1otSg-aQcVStkRhA-iCM8",
             }
         )
-        assert b"Hello world!" == ctx.decode(encoded, priv_key, context={"alg": "A128GCM"})
+        assert b"Hello world!" == recipient.decode(encoded, priv_key, context={"alg": "A128GCM"})
 
     def test_cose_usage_examples_cose_signature1(self):
 
+        # The sender side:
         priv_key = COSEKey.from_jwk(
             {
                 "kty": "EC",
@@ -542,9 +797,10 @@ class TestCOSESample:
                 "d": "V8kgd2ZBRuh2dgyVINBUqpPDr7BOMGcF22CQMIUHtNM",
             }
         )
-        ctx = COSE.new(alg_auto_inclusion=True, kid_auto_inclusion=True)
-        encoded = ctx.encode_and_sign(b"Hello world!", priv_key)
+        sender = COSE.new(alg_auto_inclusion=True, kid_auto_inclusion=True)
+        encoded = sender.encode_and_sign(b"Hello world!", priv_key)
 
+        # The recipient side:
         pub_key = COSEKey.from_jwk(
             {
                 "kty": "EC",
@@ -554,28 +810,32 @@ class TestCOSESample:
                 "y": "IBOL-C3BttVivg-lSreASjpkttcsz-1rb7btKLv8EX4",
             }
         )
-        assert b"Hello world!" == ctx.decode(encoded, pub_key)
+        recipient = COSE.new()
+        assert b"Hello world!" == recipient.decode(encoded, pub_key)
 
-        ctx = COSE.new()
-        encoded2 = ctx.encode_and_sign(
+        # variation
+        sender = COSE.new()
+        encoded2 = sender.encode_and_sign(
             b"Hello world!",
             priv_key,
             protected={"alg": "ES256"},
             unprotected={"kid": "01"},
         )
-        assert b"Hello world!" == ctx.decode(encoded2, pub_key)
+        assert b"Hello world!" == recipient.decode(encoded2, pub_key)
 
-        encoded3 = ctx.encode_and_sign(
+        encoded3 = sender.encode_and_sign(
             b"Hello world!",
             priv_key,
             protected={1: -7},
             unprotected={4: b"01"},
         )
-        assert b"Hello world!" == ctx.decode(encoded3, pub_key)
+        assert b"Hello world!" == recipient.decode(encoded3, pub_key)
 
     def test_cose_usage_examples_cose_encrypt_ecdh_aes_key_wrap(self):
 
         enc_key = COSEKey.from_symmetric_key(alg="A128GCM")
+
+        # The sender side:
         r = Recipient.from_jwk(
             {
                 "kty": "EC",
@@ -594,12 +854,15 @@ class TestCOSESample:
             }
         )
         r.apply(enc_key, recipient_key=pub_key, context={"alg": "A128GCM"})
-        ctx = COSE.new(alg_auto_inclusion=True)
-        encoded = ctx.encode_and_encrypt(
+        sender = COSE.new(alg_auto_inclusion=True)
+        encoded = sender.encode_and_encrypt(
             b"Hello world!",
             key=enc_key,
             recipients=[r],
         )
+
+        # The recipient side:
+        recipient = COSE.new()
         priv_key = COSEKey.from_jwk(
             {
                 "kty": "EC",
@@ -611,10 +874,11 @@ class TestCOSESample:
                 "d": "r_kHyZ-a06rmxM3yESK84r1otSg-aQcVStkRhA-iCM8",
             }
         )
-        assert b"Hello world!" == ctx.decode(encoded, priv_key, context={"alg": "A128GCM"})
+        assert b"Hello world!" == recipient.decode(encoded, priv_key, context={"alg": "A128GCM"})
 
     def test_cose_usage_examples_cose_signature(self):
 
+        # The sender side:
         signer = Signer.from_jwk(
             {
                 "kty": "EC",
@@ -625,9 +889,11 @@ class TestCOSESample:
                 "d": "V8kgd2ZBRuh2dgyVINBUqpPDr7BOMGcF22CQMIUHtNM",
             },
         )
-        ctx = COSE.new()
-        encoded = ctx.encode_and_sign(b"Hello world!", signers=[signer])
+        sender = COSE.new()
+        encoded = sender.encode_and_sign(b"Hello world!", signers=[signer])
 
+        # The recipient side:
+        recipient = COSE.new()
         pub_key = COSEKey.from_jwk(
             {
                 "kty": "EC",
@@ -637,8 +903,9 @@ class TestCOSESample:
                 "y": "IBOL-C3BttVivg-lSreASjpkttcsz-1rb7btKLv8EX4",
             }
         )
-        assert b"Hello world!" == ctx.decode(encoded, pub_key)
+        assert b"Hello world!" == recipient.decode(encoded, pub_key)
 
+        # variation
         signer = Signer.new(
             cose_key=COSEKey.from_jwk(
                 {
@@ -653,8 +920,8 @@ class TestCOSESample:
             protected={"alg": "ES256"},
             unprotected={"kid": "01"},
         )
-        encoded2 = ctx.encode_and_sign(b"Hello world!", signers=[signer])
-        assert b"Hello world!" == ctx.decode(encoded2, pub_key)
+        encoded2 = sender.encode_and_sign(b"Hello world!", signers=[signer])
+        assert b"Hello world!" == recipient.decode(encoded2, pub_key)
 
         signer = Signer.new(
             cose_key=COSEKey.from_jwk(
@@ -670,5 +937,5 @@ class TestCOSESample:
             protected={1: -7},
             unprotected={4: b"01"},
         )
-        encoded3 = ctx.encode_and_sign(b"Hello world!", signers=[signer])
-        assert b"Hello world!" == ctx.decode(encoded3, pub_key)
+        encoded3 = sender.encode_and_sign(b"Hello world!", signers=[signer])
+        assert b"Hello world!" == recipient.decode(encoded3, pub_key)
